###########################################################
## R koden brukt i "MATIA sier... ", 12. oktober 2022, av
## Genaro Sucarrat og Kyrre Aamdal
##
## Lisens: GNU GPL 2 eller hyere
##
## Sitering av kode:
## Genaro Sucarrat (2022): "R koden til 'MATIA sier...'",
## 12. oktober 2022. https://www.sucarrat.net/matia/
##
## R koden i denne filen gjennomfrer utregningene og de
## statistiske analysene i rapporten, og genererer figurene
## og tabellene.
##
## Innhold:
##
## 1 INITIR
## 2 RLIGE DATA: LAST OG TRANSFORMR DATA, LAG GRAFER MM.
## 3 KVARTALSDATA: LAST OG TRANSFORMR DATA, LAG GRAFER MM.
## 4 LAG PROGNOSER, LAG PROGNOSEGRAFER
## 5 LAG PROGNOSETABELLENE (LaTeX kode)
##
###########################################################


###########################################################
## 1 INITIR
###########################################################

##definer arbeidskatalog p lokal maskin:
setwd("C:/Users/sucarrat/Documents/projects/matia/2022/")
#setwd(choose.dir())

##slett alle objekter i gjeldende arbeidsrom:
rm(list = ls())

##last flgende CRAN pakker:
library(gets)   #trenger arx()
library(garchx) #trenger gdiff og glag
library(xtable) #for generering av LaTeX kode

##sett noen parametre:
iaar <- 2022 #i r
iaarTxt <- as.character(iaar)
ifjorTxt <- as.character(iaar-1)


###########################################################
## 2 RLIGE DATA: LAST OG TRANSFORMR DATA, LAG GRAFER MM.
###########################################################

##last faktiske verdier:
##======================

##last datasett m/faktiske verdier:
actualDataY <-
  read.csv("./analysis/data/actual-data-yearly-2022m9vintage.csv")
rownames(actualDataY) <- as.character(actualDataY$aar)

##hvilke serier?:
seriesNames <- c("bnpfast", "privatkons", "offkons", "investbrutto",
  "sysselsetting", "ledighetaku", "lonnsvekst", "kpi", 
  "nibor3m", "i44", "brentblend")

##lag liste med faktiske serier i zoo-format:
seriesListY <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataY[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=actualDataY[,"aar"])
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra lgarch pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataY[ ,"nokusd" ]
    nokusd <- zoo(nokusd, order.by=actualDataY[,"aar"])
    x <- x*nokusd
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra lgarch pakka
  }
  seriesListY[[i]] <- x
}
names(seriesListY) <- seriesNames


##ar(1) modeller, prognosegrafer:
##===============================

##utvalg for ar(1) modellen: bestem startr
##(delvis fra Prognoseprisen 2013)
startaar <- c(1971, #bnpfast
  1971, #privatkons
  1971, #offkons, endre til 1993 eller deromkring?
  1971, #investbrutto
  1971, #sysselsetting
  1983, #ledighetaku
  1996, #lonnsvekst
  1990, #kpi
  1992, #nibor3m; fr 1993 var renten hy (over 10%), og 1992 mistes pga. lagging
  1991, #i44
  1987) #brentblend; basert p frste data punkt i actualDataY

##matrise med ar(1) beregninger:
ar1matriseY <- matrix(NA, length(seriesNames), 7)
colnames(ar1matriseY) <-
  c("phi0","pval1","phi1", "pval2", "ser", "utvalg", "T")
rownames(ar1matriseY) <- seriesNames
ar1matriseY <- as.data.frame(ar1matriseY)

##lag matrise med R-i-andre:
mRsquareds <- matrix(NA, length(seriesNames), 1)
rownames(mRsquareds) <- rownames(ar1matriseY)

##lkke:
for(i in 1:length(seriesNames)){

  ##ar(1) beregninger:
  y <- window(seriesListY[[i]], start=startaar[i])
  yar1 <- arx(y, mc=TRUE, ar=1)
  ar1matriseY[i,"phi0"] <- yar1$mean.results["mconst","coef"]
  ar1matriseY[i,"pval1"] <- yar1$mean.results["mconst","p-value"]
  ar1matriseY[i,"phi1"] <- yar1$mean.results["ar1","coef"]  
  ar1matriseY[i,"pval2"] <- yar1$mean.results["ar1","p-value"]
  ar1matriseY[i,"ser"] <- sigma(yar1)
  ar1matriseY[i,"utvalg"] <- paste0(yar1$aux$y.index[1],
    " -- ", yar1$aux$y.index[yar1$aux$y.n])
  ar1matriseY[i,"T"] <- yar1$aux$y.n

  ##noter R-i-andre:
  mRsquareds[i,] <- rsquared(yar1)
    
}  ##slutten til lkke

##lag bedre radnavn for ar1matriseY:
rownames(ar1matriseY) <- c("BNP (fast)", "KON", "OFF", "INV (brutto)",
  "SYS", "LED (AKU)", "LON", "KPI", "NIBOR3M", "I44", "OLJE")

##lag grafer av faktiske verdier:
##===============================

ablineList <- list()
ablineList[[1]] <- c(-2,0,2,4,6) #bnpfast
ablineList[[2]] <- c(-5,0,5) #privatkons
ablineList[[3]] <- c(0,2,4,6) #offkons
ablineList[[4]] <- c(-10,-5,0,5,10,15) #investbrutto
ablineList[[5]] <- c(-3,-2,-1,0,1,2,3,4) #sysselsetting
ablineList[[6]] <- c(2,3,4,5,6) #ledighetaku
ablineList[[7]] <- c(0,2,3,4,5,6) #lonnsvekst
ablineList[[8]] <- c(1,2,3,4) #kpi
ablineList[[9]] <- c(0,2,4,6,8,10,12) #nibor3m
ablineList[[10]] <- c(-5,0,5,10) #i44
ablineList[[11]] <- c(-20,0,20,40,60,80) #brentblend

legendList <- list()
legendList[[1]] <- "bottomleft"   #bnpfast
legendList[[2]] <- "topright"     #privatkons
legendList[[3]] <- "topright"     #offkons
legendList[[4]] <- "bottomright"  #investbrutto
legendList[[5]] <- "bottomright"  #sysselsetting
legendList[[6]] <- "topright"     #ledighetaku
legendList[[7]] <- "bottomleft"   #kpi
legendList[[8]] <- "bottomleft"   #lonnsvekst
legendList[[9]] <- "topright"     #i44
legendList[[10]] <- "topleft"     #nibor3m
legendList[[11]] <- "topright"    #brentblend

legendNames <- c("BNP-vekst i %", "Konsumvekst i %",
  "Offentlig vekst i %", "Investeringsvekst i %",
  "Sysselsettingsvekst i %", "Ledighet (AKU) i %",
  "Lnnsvekst i %", "Inflasjon (KPI) i %", 
  "Renteniv (NIBOR3M)", "Valutakursvekst (I44) i %", 
  "Oljeprisvekst i %")

##lkke (lag grafer):
for(i in 1:length(seriesListY)){

  y <- window(seriesListY[[i]], start=startaar[i])
  snitt <- mean(y, na.rm=TRUE)
  autokorr1 <- ar1matriseY[i,"phi1"]  
  windows(15,9)
  
  ##get current par-values:
  def.par <- par(no.readonly=TRUE)

  ##margins (default=5.1,4.1,4.1,2.1):
  par(mar = c(2,4.1,0.5,2.1))
  
  ##plot:
  plot(y, las=1, main="", xlab="", ylab="%", cex.lab=1.4,
    col="blue", lty="solid", lwd=3, cex.axis=1.5)
  abline(h=ablineList[[i]], col="grey", lty="dashed")
  abline(h=snitt, lty="dotted",col="red",lwd=2)
  axis(side=4, at=snitt, labels = round(snitt,2), cex.axis=1.5)
  legend(legendList[[i]],
    legend=c(legendNames[i], paste0("Snitt = ", round(snitt,2), "%"),
    paste0("AR(1) = ", round(autokorr1,2)) ), cex=1.5,
    lty=c("solid","dotted","dotted"), lwd=c(3,2,2),
    col=c("blue","red","white"), bty="n")

  ##save plot:
  desttxt <-
    paste0("./report/figures/actual-yearly-", seriesNames[i], ".pdf")
  savePlot(desttxt, type="pdf")

  ##return to old par-values:
  par(def.par)

  ##turn off graphics:
  graphics.off()

}


###########################################################
## 3 KVARTALSDATA: LAST OG TRANSFORMR DATA, LAG GRAFER MM.
###########################################################

##last datasett m/faktiske verdier:
##=================================

actualDataQ<-
  read.csv("./analysis/data/actual-data-quarterly-2022m9vintage.csv")
indxQ1 <- seq(1, NROW(actualDataQ), by=4) #Q1 positions
actualDataQ1 <- actualDataQ[indxQ1,] #Q1 data
indxQ2 <- seq(2, NROW(actualDataQ), by=4) #Q2 positions
actualDataQ2 <- actualDataQ[indxQ2,] #Q2 data
##for "ekstraanalysen":
indxQ3 <- seq(3, NROW(actualDataQ), by=4) #Q3 positions
actualDataQ3 <- actualDataQ[indxQ3,] #Q3 data
indxQ4 <- seq(4, NROW(actualDataQ), by=4) #Q4 positions
actualDataQ4 <- actualDataQ[indxQ4,] #Q4 data

##hvilke serier?:
seriesNames <- c("bnpfast", "privatkons", "offkons", "investbrutto",
  "sysselsetting", "ledighetaku", "lonnsvekst", "kpi", "nibor3m",
  "i44", "brentblend")


##lag lister med faktiske serier i zoo-format:
##============================================

##Q1 data:
seriesListQ1 <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataQ1[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=1970:iaar)
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataQ1[ ,"nokusd" ]
    x <- x*nokusd
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  seriesListQ1[[i]] <- x
}
names(seriesListQ1) <- seriesNames

##Q2 data:
seriesListQ2 <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataQ2[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=1970:iaar)
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataQ2[ ,"nokusd" ]
    x <- x*nokusd
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  seriesListQ2[[i]] <- x
}
names(seriesListQ2) <- seriesNames

##Q3 data (for "ekstraanalyse"):
seriesListQ3 <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataQ3[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=1970:iaar)
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataQ3[ ,"nokusd" ]
    x <- x[-length(x)] #remove last obs
    x <- x*nokusd
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  seriesListQ3[[i]] <- x
}
names(seriesListQ3) <- seriesNames

##Q4 data (for "ekstraanalyse"):
seriesListQ4 <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataQ4[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=1970:iaar)
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataQ4[ ,"nokusd" ]
    x <- x[-length(x)] #remove last obs
    x <- x*nokusd
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra garchx pakka
  }
  seriesListQ4[[i]] <- x
}
names(seriesListQ4) <- seriesNames


##ekstraanalyse (ikke nevnt i rapporten):
##hvilken rlig kvartalsvekst bidrar mest til den
##totale rsveksten i BNP?
##===============================================

BNPY <- window(seriesListY[["bnpfast"]], start=1970, end=2021)
BNPQ1 <- window(seriesListQ1[["bnpfast"]], start=1970, end=2021)
BNPQ2 <- window(seriesListQ2[["bnpfast"]], start=1970, end=2021)
BNPQ3 <- window(seriesListQ3[["bnpfast"]], start=1970, end=2021)
BNPQ4 <- window(seriesListQ4[["bnpfast"]], start=1970, end=2021)
xtramod1 <-
  arx(BNPY, mc=FALSE, mxreg=cbind(BNPQ1, BNPQ2, BNPQ3, BNPQ4))
xtramod1
plot(xtramod)


##ar(1) modeller:
##===============

##utvalg for ar(1) modellen: bestem startr
##(samme som for rlige data)
startaar <- c(1971, #bnpfast
  1971, #privatkons
  1971, #offkons
  1971, #investbrutto
  1971, #sysselsetting
  1983, #ledighetaku
  1996, #lonnsvekst
  1990, #kpi
  1992, #nibor3m; fr 1993 var renten hy (over 10%), og 1992 mistes pga. lagging
  1991, #i44
  1987) #brentblend; basert p frste data punkt i actualDataY

##lag matriser til ar(1) beregningene:
ar1matriseQ1 <- matrix(NA, length(seriesNames), 7)
colnames(ar1matriseQ1) <- c("phi0","pval1","phi1", "pval2",
  "ser", "utvalg", "T")
rownames(ar1matriseQ1) <- seriesNames
ar1matriseQ1 <- as.data.frame(ar1matriseQ1)
ar1matriseQ2 <- ar1matriseQ1

##Q1 data: lkke
for(i in 1:length(seriesNames)){

  ##ar(1) beregninger:
  y <- window(seriesListQ1[[i]], start=startaar[i])
  yar1 <- arx(y, mc=TRUE, ar=1)
  ar1matriseQ1[i,"phi0"] <- yar1$mean.results["mconst","coef"]
  ar1matriseQ1[i,"pval1"] <- yar1$mean.results["mconst","p-value"]
  ar1matriseQ1[i,"phi1"] <- yar1$mean.results["ar1","coef"]
  ar1matriseQ1[i,"pval2"] <- yar1$mean.results["ar1","p-value"]
  ar1matriseQ1[i,"ser"] <- sigma(yar1)
  ar1matriseQ1[i,"utvalg"] <- paste0(yar1$aux$y.index[1],
    " -- ", yar1$aux$y.index[yar1$aux$y.n])
  ar1matriseQ1[i,"T"] <- yar1$aux$y.n
  
}  ##slutten til lkke Q1 data

##Q1 data: lag bedre radnavn for ar1matrise:
rownames(ar1matriseQ1) <- c("BNP vekst (fastland)", "Konsumvekst",
  "Offentlig vekst", "Investeringsvekst (brutto)",
  "Sysselsettingsvekst", "Ledighet (AKU)", "Lonnsvekst",
  "Inflasjon (KPI)", "Renteniva (NIBOR3M)", "Valutakursvekst (I44)",
  "Oljeprisvekst (NOK)")

##Q2 data: lkke
for(i in 1:length(seriesNames)){

  ##ar(1) beregninger:
  y <- window(seriesListQ2[[i]], start=startaar[i])
  yar1 <- arx(y, mc=TRUE, ar=1)
  ar1matriseQ2[i,"phi0"] <- yar1$mean.results["mconst","coef"]
  ar1matriseQ2[i,"pval1"] <- yar1$mean.results["mconst","p-value"]
  ar1matriseQ2[i,"phi1"] <- yar1$mean.results["ar1","coef"]
  ar1matriseQ2[i,"pval2"] <- yar1$mean.results["ar1","p-value"]
  ar1matriseQ2[i,"ser"] <- sigma(yar1)
  ar1matriseQ2[i,"utvalg"] <- paste0(yar1$aux$y.index[1],
    " -- ", yar1$aux$y.index[yar1$aux$y.n])
  ar1matriseQ2[i,"T"] <- yar1$aux$y.n
  
}  ##slutten til lkke

##Q2 data: lag bedre radnavn for ar1matrise:
rownames(ar1matriseQ2) <- c("BNP vekst (fastland)", "Konsumvekst",
  "Offentlig vekst", "Investeringsvekst (brutto)",
  "Sysselsettingsvekst", "Ledighet (AKU)", "Lonnsvekst",
  "Inflasjon (KPI)", "Renteniva (NIBOR3M)", "Valutakursvekst (I44)",
  "Oljeprisvekst (NOK)")

##Q1 data: lag grafer av faktiske verdier:
##========================================

ablineList <- list()
ablineList[[1]] <- c(-4,-2,0,2,4,6,8) #bnpfast
ablineList[[2]] <- c(-4,-2,0,2,4,6,8) #privatkons
ablineList[[3]] <- c(-4,-2,0,2,4,6,8,10) #offkons
ablineList[[4]] <- c(-10,-5,0,5,10,15,20) #investbrutto
ablineList[[5]] <- c(-1,0,1,2,3,4) #sysselsetting
ablineList[[6]] <- c(2,3,4,5,6) #ledighetaku
ablineList[[7]] <- c(0,2,3,4,5,6) #lonnsvekst
ablineList[[8]] <- c(-1,0,1,2,3,4) #kpi
ablineList[[9]] <- c(0,2,4,6,8,10) #nibor3m
ablineList[[10]] <- c(-5,0,5,10) #i44
ablineList[[11]] <- c(-50,0,50,100,150) #brentblend
names(ablineList) <- seriesNames

legendList <- list()
legendList[[1]] <- "bottomleft"   #bnpfast
legendList[[2]] <- "bottomleft"   #privatkons
legendList[[3]] <- "bottomleft"   #offkons
legendList[[4]] <- "topright"     #investbrutto
legendList[[5]] <- "bottomleft"   #sysselsetting
legendList[[6]] <- "topright"     #ledighetaku
legendList[[7]] <- "bottomleft"   #lonnsvekst
legendList[[8]] <- "bottomleft"   #kpi
legendList[[9]] <- "topright"     #nibor3m
legendList[[10]] <- "topleft"     #i44
legendList[[11]] <- "topright"    #brentblend
names(legendList) <- seriesNames

legendNames <- c("BNP-vekst i %", "Konsumvekst i %",
  "Offentlig vekst i %", "Investeringsvekst i %",
  "Sysselsettingsvekst i %", "Ledighet (AKU) i %",
  "Lnnsvekst i %", "Inflasjon (KPI) i %",
  "Renteniv (NIBOR3M)", "Valutakursvekst (I44) i %",
  "Oljeprisvekst i %")

##lkke (lag grafer):
for(i in 1:length(seriesListQ1)){

  y <- window(seriesListQ1[[i]], start=startaar[i])
  snitt <- mean(y, na.rm=TRUE)
  autokorr1 <- ar1matriseQ1[i,"phi1"]  
  windows(15,9)
  
  ##get current par-values:
  def.par <- par(no.readonly=TRUE)

  ##margins (default=5.1,4.1,4.1,2.1):
  par(mar = c(2,4.1,0.5,2.1))
  
  ##plot:
  plot(y, las=1, main="", xlab="", ylab="%", cex.lab=1.4,
    col="blue", lty="solid", lwd=3, cex.axis=1.5)
  abline(h=ablineList[[i]], col="grey", lty="dashed")
  abline(h=snitt, lty="dotted",col="red",lwd=2)
  axis(side=4, at=snitt, labels = round(snitt,2), cex.axis=1.5)
  legend(legendList[[i]],
    legend=c(legendNames[i], paste0("Snitt = ", round(snitt,2), "%"),
    paste0("AR(1) = ", round(autokorr1,2)) ), cex=1.5,
    lty=c("solid","dotted","dotted"), lwd=c(3,2,2),
    col=c("blue","red","white"), bty="n")

  ##save plot:
  desttxt <-
    paste0("./report/figures/actual-quarterlyQ1-", seriesNames[i], ".pdf")
  savePlot(desttxt, type="pdf")

  ##return to old par-values:
  par(def.par)

  ##turn off graphics:
  graphics.off()

}


##Q2 data: lag grafer av faktiske verdier:
##========================================

ablineList <- list()
ablineList[[1]] <- c(-5,0,5,10) #bnpfast
ablineList[[2]] <- c(-10,-5,0,5,10) #privatkons
ablineList[[3]] <- c(-2,0,2,4,6,8) #offkons
ablineList[[4]] <- c(-10,0,10,20) #investbrutto
ablineList[[5]] <- c(-2,-1,0,1,2,3,4) #sysselsetting
ablineList[[6]] <- c(2,3,4,5,6) #ledighetaku
ablineList[[7]] <- c(0,2,3,4,5,6) #lonnsvekst
ablineList[[8]] <- c(1,2,3,4,5,6) #kpi
ablineList[[9]] <- c(0,2,4,6,8,10) #nibor3m
ablineList[[10]] <- c(-10,-5,0,5,10) #i44
ablineList[[11]] <- c(-50,0,50,100) #brentblend
names(ablineList) <- seriesNames

legendList <- list()
legendList[[1]] <- "bottomleft"   #bnpfast
legendList[[2]] <- "bottomleft"   #privatkons
legendList[[3]] <- "bottomleft"   #offkons
legendList[[4]] <- "topright"     #investbrutto
legendList[[5]] <- "bottomleft"   #sysselsetting
legendList[[6]] <- "topright"     #ledighetaku
legendList[[7]] <- "bottomleft"   #lonnsvekst
legendList[[8]] <- "bottomleft"   #kpi
legendList[[9]] <- "topright"     #nibor3m
legendList[[10]] <- "topleft"     #i44
legendList[[11]] <- "topright"    #brentblend
names(legendList) <- seriesNames

legendNames <- c("BNP-vekst i %", "Konsumvekst i %",
  "Offentlig vekst i %", "Investeringsvekst i %",
  "Sysselsettingsvekst i %", "Ledighet (AKU) i %",
  "Lnnsvekst i %", "Inflasjon (KPI) i %", 
  "Renteniv (NIBOR3M)", "Valutakursvekst (I44) i %",
  "Oljeprisvekst i %")

##lkke (lag grafer):
for(i in 1:length(seriesListQ2)){

  y <- window(seriesListQ2[[i]], start=startaar[i])
  snitt <- mean(y, na.rm=TRUE)
  autokorr1 <- ar1matriseQ2[i,"phi1"]  
  windows(15,9)
  
  ##get current par-values:
  def.par <- par(no.readonly=TRUE)

  ##margins (default=5.1,4.1,4.1,2.1):
  par(mar = c(2,4.1,0.5,2.1))
  
  ##plot:
  plot(y, las=1, main="", xlab="", ylab="%", cex.lab=1.4,
    col="blue", lty="solid", lwd=3, cex.axis=1.5)
  abline(h=ablineList[[i]], col="grey", lty="dashed")
  abline(h=snitt, lty="dotted",col="red",lwd=2)
  axis(side=4, at=snitt, labels = round(snitt,2), cex.axis=1.5)
  legend(legendList[[i]],
    legend=c(legendNames[i], paste0("Snitt = ", round(snitt,2), "%"),
    paste0("AR(1) = ", round(autokorr1,2)) ), cex=1.5,
    lty=c("solid","dotted","dotted"), lwd=c(3,2,2),
    col=c("blue","red","white"), bty="n")

  ##save plot:
  desttxt <-
    paste0("./report/figures/actual-quarterlyQ2-", seriesNames[i], ".pdf")
  savePlot(desttxt, type="pdf")

  ##return to old par-values:
  par(def.par)

  ##turn off graphics:
  graphics.off()

}


###########################################################
## 4 LAG PROGNOSER, LAG PROGNOSEGRAFER
###########################################################
                        
##hvilke serier?
seriesNames <- c("bnpfast", "privatkons", "offkons",
  "investbrutto", "sysselsetting", "ledighetaku", "lonnsvekst",
  "kpi", "nibor3m", "i44", "brentblend")

##FIN prognoser (Nasjonalbudsjettet):
##===================================

##datasettet med FIN prognoser:
forecastData <- read.csv("./analysis/data/forecasts-nb2023.csv")
rownames(forecastData) <- as.character(forecastData$aar)

##matrise med FIN prognoser:
FINprognoser <- matrix(NA, 3, length(seriesNames))
colnames(FINprognoser) <- seriesNames
rownames(FINprognoser) <- rownames(forecastData)

##fyll matrisen FINprognoser:
for(i in 1:length(seriesNames)){

  ##FIN prognoser:
  if( seriesNames[i] %in% setdiff(seriesNames,c("i44","brentblend")) ){
    FINseriesName <- paste0("FIN", seriesNames[i])
    x <- forecastData[, FINseriesName ]
  }else{
    if( seriesNames[i]=="i44" ){
      FINseriesName <- "FINi44vekst"
      x <- forecastData[, FINseriesName ]
    }
    if( seriesNames[i]=="brentblend" ){
      x <- forecastData[, "FINbrentblendnok" ]
      x <- ##ifjor = budsjettr - 2
        c( actualDataY[ ifjorTxt, "brentblend"] * actualDataY[ ifjorTxt, "nokusd"], x)
      x <- 100*gdiff(x)/glag(x) #gdiff, glag: fra lgarch pakka
      x <- x[-1] #fjern frste verdi
    }
  } 
  FINprognoser[,i] <- x

}  


##beregn modeller, lag prognosegrafer:
##====================================

##last spesialutviklet kode:
source("./analysis/code/predictFun.R")
source("./analysis/code/quantileFun.R")
source("./analysis/code/predictPlot.R")

##bestem startr (delvis fra Prognoseprisen 2013)
##til beregningsutvalget ("in-sample") som skal brukes
startaar <- c(1971, #bnpfast         
  1971, #privatkons
  1971, #offkons
  1971, #investbrutto
  1971, #sysselsetting
  1983, #ledighetaku
  1996, #lonnsvekst
  1990, #kpi
  1992, #nibor3m      ## Fr 1993 var renten hy (over 10%), og 1992 mistes pga. lagging
  1991, #i44
  1987) #brentblend   ## Basert p frste data punkt i actualDataY

##hlines liste (til bruk i prognosegrafer):
hlinesList <- list()
hlinesList[[1]] <- c(-2,0,2,4,6) #bnpfast
hlinesList[[2]] <- c(-5,0,5,10) #privatkons
hlinesList[[3]] <- c(0,1,2,3,4,5,6) #offkons
hlinesList[[4]] <- c(-5,0,5,10) #investbrutto
hlinesList[[5]] <- c(-2,0,2,4,6) #sysselsetting
hlinesList[[6]] <- c(2.5,3,3.5,4,4.5,5,5.5) #ledighetaku
hlinesList[[7]] <- c(2,3,4,5,6) #lonnsvekst
hlinesList[[8]] <- c(1,,2,3,4,5) #kpi
hlinesList[[9]] <- c(0,1,2,3,4,5) #nibor3m
hlinesList[[10]] <- c(-10,-5,0,5,10) #i44
hlinesList[[11]] <- c(-40,-20,0,20,40,60,80) #brentblend
names(hlinesList) <- seriesNames

##matrise m/beregninger: blanda frekvens (BF)
eq1results <- matrix(NA, length(seriesNames), 11)
colnames(eq1results) <- c("phi0", "pval0", "phi1", "pval1",
  "phi2", "pval2", "phi3", "pval3", "ser", "utvalg", "T")
rownames(eq1results) <- seriesNames
eq1results <- as.data.frame(eq1results)

##lag matrise med R-i-andre til BF-modeller:
mRsquaredsBF <- matrix(NA, length(seriesNames), 1)
rownames(mRsquaredsBF) <- rownames(eq1results)

##matrise m/beregninger: ar(1) modeller (rlige data)
eq2results <- eq1results

##MATIA prognoser og kvantiler:
MATIAprognoser <- list()

##for reproduserbarhet (generering av kvantiler):
set.seed(123)

##lkke:
for(i in 1:length(seriesNames)){

  ##Y1t: yearly data
  Y1t <- seriesListY[[ seriesNames[i] ]] ##yearly data
  t1 <- as.numeric(index(Y1t)[1])
  t2 <- iaar
  Y1t <- zoo(c(coredata(Y1t),NA), order.by=t1:t2)

  ##X1t, X2t: quarterly data
  X1t <- seriesListQ1[[i]] ##Q1 data
  X2t <- seriesListQ2[[i]] ##Q2 data
  
  ##transformr og tidsforskyv variabler:
  Y1t <- window(Y1t, start=startaar[i])
  Y1tmin1 <- glag(Y1t)
  X1t <- window(X1t, start=startaar[i])
  X2t <- window(X2t, start=startaar[i])

  ##beregn blanda frekvens modell:
  eq1 <- arx(Y1t, mc=TRUE, mxreg=cbind(Y1tmin1, X1t, X2t),
    qstat.options=c(2,2))                              

  ##beregn ar(1) modell:
  eq2 <- arx(Y1t, mc=TRUE, mxreg=cbind(Y1tmin1),
    qstat.options=c(2,2))

  ##forbered prognosemaking:
  mY <- cbind(Y1t, X1t, X2t)
  pars1 <- as.numeric(coef(eq1))
  pars2 <- as.numeric(coef(eq2))
  resid1 <- residuals(eq1)
  resid2 <- residuals(eq2)
  
  ##generr prognoser:
  mYhat <- predictFun(mY, pars1, pars2)

  ##generr kvantiler:
  set.seed(123) #for re-produserbarhet
  mY1quantiles <- quantileFun(mY, pars1, pars2, resid1, resid2,
      probs=c(0.10, 0.25, 0.75, 0.90), n.ahead=3, n.sim=20000)

  ##lagre eq1 resultater:
  eq1results[i,c(1,3,5,7)] <- eq1$mean.results[,"coef"]
  eq1results[i,c(2,4,6,8)] <- eq1$mean.results[,"p-value"]
  eq1results[i,"ser"] <- sigma(eq1)
  eq1results[i,"utvalg"] <- paste0(eq1$aux$y.index[1],
    " -- ", eq1$aux$y.index[eq1$aux$y.n])
  eq1results[i,"T"] <- eq1$aux$y.n

  ##lagre eq2 resultater:
  eq2results[i,c(1,3)] <- eq2$mean.results[,"coef"]
  eq2results[i,c(2,4)] <- eq2$mean.results[,"p-value"]
  eq2results[i,"ser"] <- sigma(eq2)
  eq2results[i,"utvalg"] <- paste0(eq2$aux$y.index[1],
    " -- ", eq2$aux$y.index[eq2$aux$y.n])
  eq2results[i,"T"] <- eq2$aux$y.n

  ##lagre R-i-andre til BF-modeller:
  mRsquaredsBF[i,] <- rsquared(eq1)
    
  ##lagre anslag og kvantiler:
  yhat <- mYhat[6:8,"Y1hat"]
  MATIAprognoser[[i]] <- cbind(yhat, mY1quantiles)
    
  ##plot.options:
  FaktiskNBtxt <- paste0("Faktisk | Nasjonalbudsjettet ",
    as.character(iaar+1))
  plot.options <- list(lwd=c(2,2), lty=c(1,1), ylab="%",
    col=c("red","blue"), ##colours. Order: col=c(Forecast,Actual)
    legend.text=c("MATIA", FaktiskNBtxt),
    dot.at.origin=TRUE, line.at.origin=TRUE,
    start.at.origin=TRUE, hlines=hlinesList[[i]])

  ##lag prognosegraf:
  windows(6,4) ##set graphics windows size; max is (15,9)?
  predictPlot(mYhat[,1], mY1quantiles, FINprognoser[,i], plot.options)
  desttxt <- ##lagre graf:
    paste0("./report/figures/forecast-", seriesNames[i], ".pdf")
  savePlot(desttxt, type="pdf")
  graphics.off() ##turn off graphics:

} ##slutten til lkke

##sett navn p elementene i MATIAprognoser:
names(MATIAprognoser) <- seriesNames

##lag kortere radnavn for eq1 og eq2 matriser:
rownames(eq1results) <- c("BNP (fast)", "KON", "OFF", "INV (brutto)",
  "SYS", "LED (AKU)", "LON", "KPI", "NIBOR3M", "I44", "OLJE")
rownames(eq2results) <- rownames(eq1results)

##Blanda Frekvens (BF) likning, LaTeX kode:
matriseTex <- xtable(eq1results,
  align=c("l","c","c","c","c","c","c","c","c","c","c","c"), digits=2)
print(matriseTex,
    floating=FALSE, ##FALSE: removes table environment
    format.args=list(decimal.mark=","), ##comma instead of dot
    math.style.negative=TRUE, ##use $-$ for negative sign
    NA.string="--"
    )
    
##AR(1) likning, LaTeX kode:
matriseTex <- xtable(eq2results,
  align=c("l","c","c","c","c","c","c","c","c","c","c","c"), digits=2)
print(matriseTex,
    floating=FALSE, ##FALSE: removes table environment
    format.args=list(decimal.mark=","), ##comma instead of dot
    math.style.negative=TRUE, ##use $-$ for negative sign
    NA.string="--"
    )


###########################################################
## 5 LAG PROGNOSETABELLENE (LaTeX kode)
###########################################################

##lag tabell:
mTabell <- matrix(NA, 7, 3)
colnames(mTabell) <- paste0(c(iaar, iaar+1, iaar+2))
rownames(mTabell) <- c(paste0("Nasjonalbudsjettet ", iaar+1, ":"),
  "MATIA anslag:", "MATIA sannsynlighetsanslag...",
  "  ...50% vre grense:", "  ...50% nedre grense:",
  "  ...80% vre grense:", "  ...80% nedre grense:")

##tabelloverskrifter:
tabellOverskrTeX <- c("\\noindent {\\bf BNP \\%-vekst (fastland):} \\\\ \n",
  "\\noindent {\\bf Privat konsum \\%-vekst:} \\\\ \n",
  "\\noindent {\\bf Offentlig konsum \\%-vekst:} \\\\ \n",
  "\\noindent {\\bf Investeringer (brutto) \\%-vekst:} \\\\ \n",
  "\\noindent {\\bf Sysselsetting \\%-vekst:} \\\\ \n",
  "\\noindent {\\bf Ledighet (AKU) i \\%:} \\\\ \n",
  "\\noindent {\\bf Lnnsvekst (LN) i \\%:} \\\\ \n",
  "\\noindent {\\bf Inflasjon (KPI) i \\%:} \\\\ \n",
  "\\noindent {\\bf Renteniv (NIBOR3M) i \\%:} \\\\ \n",
  "\\noindent {\\bf Valutakurs (I44), endring i \\%:} \\\\ \n",
  "\\noindent {\\bf Oljepris (NOK), endring i \\%:} \\\\ \n")

##lkke:
for(i in 1:length(seriesNames)){

  ##FIN
  mTabell["Nasjonalbudsjettet 2023:",] <- FINprognoser[,i]

  ##MATIA:
  mTabell["MATIA anslag:",] <- coredata( MATIAprognoser[[i]][, "yhat" ] )
  ##50% vre grense:
  mTabell[4,] <- coredata( MATIAprognoser[[i]][, "0.75" ] )
  ##50% nedre grense:
  mTabell[5,] <- coredata( MATIAprognoser[[i]][, "0.25" ] )
  ##80% vre grense:
  mTabell[6,] <- coredata( MATIAprognoser[[i]][, "0.9" ] )
  ##80% nedre grense:
  mTabell[7,] <- coredata( MATIAprognoser[[i]][, "0.1" ] )

  #LaTeX kode:
  mTabellTex <- xtable(mTabell, align=c("l","r","r","r"), digits=1)
  cat(tabellOverskrTeX[i])
  print(mTabellTex,
    floating=FALSE, ##FALSE: removes table environment
    format.args=list(decimal.mark=","), ##comma instead of dot
    math.style.negative=TRUE ##use $-$ for negative sign
    )

}  
